#!/usr/bin/python -tt
#
# Van Assche Alphonse <alcapcom@gmail.com>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 only
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import os
import xf86config
import re
from rhpl.translate import _, N_,textdomain

##
## I18N
## 
import gettext
gettext.bindtextdomain ("system-config-xselector", "/usr/share/locale")
gettext.textdomain ("system-config-xselector")
_=gettext.gettext

# TODO make of find a cool icon
ICON_PATH = "/usr/share/system-config-xselector/pixmaps/system-config-xselector.png"
GDM_CUSTOMCONF_FILE = "/etc/gdm/custom.conf"
XORG_CONF_FILE = "/etc/X11/xorg.conf"

GDM_XGL_CONFIG_LINE = "0=xgl"
BACKUP_EXTENSION = ".backup-xselector"
XGL_DEFAULT_SERVER_DEF="""
# Experimental build of Xgl
[server-xgl]
name=Xgl server
%s
flexible=false
"""


class XselectorError(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
        return self.value

class Xselector:
    """
    I provide functions to select a "GLX Xorg implementation"
    
    FIXME i have not found the way to add or remove extensions
          with the xf86config.py module, if we can do that, _setComposite()
          function can be update and xorg.conf file will be read and
          write only once.
    """
    def __init__(self, isVerbose = False):
        self.xconfigpath = XORG_CONF_FILE
        self.icon = ICON_PATH
        self.verbose = isVerbose
 
    def enableAiglx(self):
        """Enabled Aiglx support"""
        self._setComposite(True)
        self._setAiglxFlag(True)
        self._setXglInGDM(False)
    
    def enableXgl(self):
        """Enabled Xgl support"""
        self._setComposite(False)
        self._setAiglxFlag(False)
        self._setXglInGDM(True)
        self._setGDMXglServerDef()
    
    def disableAllGlxStuff(self):
        """Disable Aiglx, Xgl and the composite extension"""
        self._setAiglxFlag(False)
        self._setComposite(False)
        self._setXglInGDM(False)
        
    def getCurrentConfig(self, gdmConfigFile=GDM_CUSTOMCONF_FILE):
        """This function return the X configuration as a string.
        Returned string: "aiglx" | "xgl" | "noglx"
        """
        gdmCustomStr = self._readFile(gdmConfigFile)
        if re.search("\n" + GDM_XGL_CONFIG_LINE, gdmCustomStr, re.IGNORECASE):
                return "xgl"
        xorgConfStr = self._readFile(self.xconfigpath)
        if re.search ("\n.*Option.*AIGLX.*ON", xorgConfStr, re.IGNORECASE ) \
                    or not re.search (".*AIGLX.*", xorgConfStr, re.IGNORECASE):
            return "aiglx"
        # If the above fails, noglx is choice.
        return "noglx"
    
    def _setAiglxFlag(self, boolValue = False):
        self._loadXconfig()
        self._verbose(_("- set aiglx flag: %s") % str(boolValue))
        if (boolValue):
            val = "on"
        else:
            val = "off"
        self.xconfig.flags = xf86config.XF86ConfFlags()
        self._get_option(self.xconfig.flags, "AIGLX").val = val
        self._writeXconfig()
   
    def _setComposite(self, boolValue = True):
        """enable or disable Composite extension"""
        strFile = self._readFile(self.xconfigpath)
        if (boolValue):
            p = re.compile('\n.*Option.*Composite.*', re.IGNORECASE)
            strFile = p.sub( '', strFile)
        else:
            p = re.compile('Section.*Extensions.*\n', re.IGNORECASE)
            if re.search(p, strFile):
                p = re.compile('Option.*Composite.*\n', re.IGNORECASE)
                strToAdd = "\tOption   \"Composite\" \"Disable\"\n"
                strFile = p.sub(strToAdd, strFile)
            else:
                extensions = "\nSection \"Extensions\"\n"
                extensions += "\tOption   \"Composite\" \"Disable\"\n"
                extensions += "EndSection\n"
                strFile += extensions
        self._verbose(_("- set composite: %s") % str(boolValue))
        self._writeFile(self.xconfigpath,strFile)
   
    def _setXglInGDM(self, boolValue = True):
        gdmCustomStr = self._readFile(GDM_CUSTOMCONF_FILE)
        self._verbose(_("- set xgl in GDM: %s") % str(boolValue))
        if (boolValue):
            if not re.search("\n" + GDM_XGL_CONFIG_LINE, gdmCustomStr):
                p = re.compile('\n\[servers\].*\n', re.IGNORECASE)
                replaceStr = "\n[servers]\n" + GDM_XGL_CONFIG_LINE + "\n"
                gdmCustomStr = p.sub(replaceStr, gdmCustomStr)
            else:
                self._verbose(_("- GDM has already a valid Xgl configuration, nothing todo."))
                return
        else:
            p = re.compile("\n" + GDM_XGL_CONFIG_LINE + ".*\n", re.IGNORECASE)
            gdmCustomStr = p.sub('\n', gdmCustomStr) 
        self._writeFile(GDM_CUSTOMCONF_FILE, gdmCustomStr)

    def _setGDMXglServerDef(self):
        # Do nothing if Xgl is not found on the system.
        vendor = self._getVendor()
        if not os.path.exists("/usr/bin/Xgl"):
            self._verbose(_("- Xgl was not found, nothing todo."))
            return
        if vendor == "fglrx":
            command="command=/usr/bin/Xgl -ac -accel glx:pbuffer -accel xv:pbuffer"
        elif vendor == "nvidia":
            command="command=/usr/bin/Xgl -ac -accel glx:pbuffer -accel xv"
        else:
            command="command=/usr/bin/Xgl -ac -accel glx:pbuffer -accel xv"
        gdmCustomStr = self._readFile(GDM_CUSTOMCONF_FILE)
        # replace all command=/usr/bin/Xgl lines.
        if re.search("\ncommand=/usr/bin/Xgl.*", gdmCustomStr):
            p = re.compile("\ncommand=/usr/bin/Xgl.*", re.IGNORECASE)
            gdmCustomStr = p.sub("\n" + command, gdmCustomStr)
        else:
            self._verbose(_("- Xgl server definition not found in %s") % GDM_CUSTOMCONF_FILE)
            self._addGDMXglServerDef(command)
            return
        self._writeFile(GDM_CUSTOMCONF_FILE, gdmCustomStr)
        
    def _addGDMXglServerDef(self, command):
        gdmCustomStr = self._readFile(GDM_CUSTOMCONF_FILE)
        if not re.search("\n\[server-xgl\]", gdmCustomStr):
            gdmCustomStr = gdmCustomStr + XGL_DEFAULT_SERVER_DEF % command
            self._verbose(_("- Xgl server definition added in %s.") % GDM_CUSTOMCONF_FILE)
        else:
            self._verbose(_("- GDM have already a Xgl server definition, nothing todo."))
            return
        self._writeFile(GDM_CUSTOMCONF_FILE, gdmCustomStr)
     
    def _writeXconfig(self):
        try:
            self.xconfig.comment = "# Xorg configuration created by system-config-xselector\n"
            # Backup the file before make any changes
            self._backupFile(self.xconfigpath)
            self._verbose(_("Write file %s") % self.xconfigpath)
            self.xconfig.write(self.xconfigpath)
        except TypeError:
            raise XselectorError(_("Cannot write %s") % self.xconfigpath)
        
    def _writeFile(self, filename, textToWrite):
        try:
            # Backup the file before make any changes
            self._backupFile(filename)
            self._verbose(_("Write file %s") % filename)
            fileHandle = open (filename, 'w')
            fileHandle.write(textToWrite)
            fileHandle.close()
        except IOError, (errno, strerror):
            raise XselectorError(_("Cannot write %s") % filename + ": " + strerror)
        
    def _readFile(self, filename):
        try:
            self._verbose(_("Read file %s") % filename)
            fileHandle = open (filename, 'r')
            text = fileHandle.read()
            fileHandle.close()
        except OSError, (errno, strerror):
            raise XselectorError(_("Cannot Read %s") % filename + " : " + strerror)
        return text

    def _backupFile(self, filename):
        try:
            self._verbose(_("Backup file %s ") % filename + " => " + filename + BACKUP_EXTENSION)
            os.rename(filename, filename + BACKUP_EXTENSION)
        except OSError, (errno, strerror):
            raise XselectorError(_("Cannot Backup %s") % filename + ': ' + strerror)
    
    def _get_option(self, device, optionname):
        res = filter (lambda o: o.name == optionname, device.options)
        if len (res) > 0:
            return res[0]
        else:
            option = xf86config.XF86Option(optionname)
            device.options.insert(option)
            return option
       
    def _loadXconfig(self):
        try:
            (self.xconfig, self.xconfigpath) = xf86config.readConfigFile()
        except:
            raise XselectorError(_("Could not load X config file"))

    def _getDriver(self):
        """Returns the current Driver tag's value"""
        for dev in self.xconfig.device:
            driver = dev.driver
        return driver

    def _getVendor(self):
        self._loadXconfig()
        return self._getDriver()
    
    def _verbose(self, str):
        if self.verbose:
            print str


